==> Botan-1.3.14.tgz: Botan-1.3.14/modules/mp_asm64/mp_madd.h <==
/*************************************************
* MPI Multiply-Add Core Header File              *
* (C) 1999-2004 The Botan Project                *
*************************************************/

#ifndef BOTAN_MP_MADD_H__
#define BOTAN_MP_MADD_H__

#include <botan/mp_core.h>

#if (BOTAN_MP_WORD_BITS != 64)
   #error The mp_asm64 module requires that BOTAN_MP_WORD_BITS == 64
#endif

namespace Botan {

/*************************************************
* Multiply-Add Operation                         *
*************************************************/
inline void bigint_madd(word a, word b, word c, word d,
                        word* out_low, word* out_high)
   {
   word z0 = 0, z1 = 0;

#if defined(__GNUC__) && defined(__alpha__)
   asm("umulh %1,%2,%0" : "=r" (z0) : "r" (a), "r" (b));
   z1 = a * b;
#elif defined(__GNUC__) && defined(__ia64__)
   asm("xmpy.hu %0=%1,%2" : "=f" (z0) : "f" (a), "f" (b));
   z1 = a * b;
#elif defined(__GNUC__) && defined(_ARCH_PPC64)
   asm("mulhdu %0,%1,%2" : "=r" (z0) : "r" (a), "r" (b) : "cc");
   z1 = a * b;
#elif defined(__GNUC__) && defined(__x86_64__)
   asm("mulq %3" : "=d" (z0), "=a" (z1) : "a" (a), "rm" (b) : "cc");
#elif defined(__GNUC__) && defined(__mips64)
   asm("dmultu %2,%3" : "=h" (z0), "=l" (z1) : "r" (a), "r" (b));
#else
   #error "Don't know how to do a 64x64->128 multiply on this system/compiler"
#endif

   z1 += c; if(z1 < c) z0++;
   z1 += d; if(z1 < d) z0++;

   *out_low = z1;
   *out_high = z0;
   }

}

#endif

==> Botan-1.4.11.tgz: Botan-1.4.11/modules/mp_asm64/mp_madd.h <==
/*************************************************
* MPI Multiply-Add Core Header File              *
* (C) 1999-2005 The Botan Project                *
*************************************************/

#ifndef BOTAN_MP_MADD_H__
#define BOTAN_MP_MADD_H__

#include <botan/mp_core.h>

namespace Botan {

/*************************************************
* Multiply-Add Operation                         *
*************************************************/
inline void bigint_madd(word a, word b, word c, word d,
                        word* out_low, word* out_high)
   {
#if (BOTAN_MP_WORD_BITS != 64)
   #error The mp_asm64 module requires that BOTAN_MP_WORD_BITS == 64
#endif

   word z0 = 0, z1 = 0;

#if defined(__GNUC__)

#if defined(BOTAN_TARGET_ARCH_IS_ALPHA)
   asm("umulh %1,%2,%0" : "=r" (z0) : "r" (a), "r" (b));
   z1 = a * b;
#elif defined(BOTAN_TARGET_ARCH_IS_IA64)
   asm("xmpy.hu %0=%1,%2" : "=f" (z0) : "f" (a), "f" (b));
   z1 = a * b;
#elif defined(BOTAN_TARGET_ARCH_IS_PPC64)
   asm("mulhdu %0,%1,%2" : "=r" (z0) : "r" (a), "r" (b) : "cc");
   z1 = a * b;
#elif defined(BOTAN_TARGET_ARCH_IS_AMD64)
   asm("mulq %3" : "=d" (z0), "=a" (z1) : "a" (a), "rm" (b) : "cc");
#elif defined(BOTAN_TARGET_ARCH_IS_MIPS64)
   asm("dmultu %2,%3" : "=h" (z0), "=l" (z1) : "r" (a), "r" (b));
#else
   bigint_wordmul(a, b, &z1, &z0);
#endif

#else
   bigint_wordmul(a, b, &z1, &z0);
#endif

   z1 += c; if(z1 < c) z0++;
   z1 += d; if(z1 < d) z0++;

   *out_low = z1;
   *out_high = z0;
   }

}

#endif
